@@ -14,8 +14,8 @@ exception, so it should hopefully not get unnoticed).
 
 """
 
-import os
 import importlib.util
+import os
 
 
 def get_annotations():
@@ -106,7 +106,7 @@ multiarray_funcs_api = {
     '__unused_indices__': (
         [1, 4, 40, 41, 66, 67, 68, 81, 82, 83,
          103, 115, 117, 122, 163, 164, 171, 173, 197,
-         201, 202, 208, 219, 220, 221, 222, 223, 278,
+         201, 202, 208, 219, 220, 221, 222, 278,
          291, 293, 294, 295, 301]
         # range/slots reserved DType classes (see _public_dtype_api_table.h):
         + list(range(320, 361)) + [366, 367, 368]
@@ -293,8 +293,8 @@ multiarray_funcs_api = {
     # Unused slot 220, was `PyArray_DatetimeToDatetimeStruct`
     # Unused slot 221, was `PyArray_TimedeltaToTimedeltaStruct`
     # Unused slot 222, was `PyArray_DatetimeStructToDatetime`
-    # Unused slot 223, was `PyArray_TimedeltaStructToTimedelta`
     # NDIter API
+    'NpyIter_GetTransferFlags':             (223, MinVersion("2.3")),
     'NpyIter_New':                          (224,),
     'NpyIter_MultiNew':                     (225,),
     'NpyIter_AdvancedNew':                  (226,),
@@ -407,6 +407,8 @@ multiarray_funcs_api = {
     # `PyDataType_GetArrFuncs` checks for the NumPy runtime version.
     '_PyDataType_GetArrFuncs':                       (365,),
     # End 2.0 API
+    # NpyIterGetTransferFlags (slot 223) added.
+    # End 2.3 API
 }
 
 ufunc_types_api = {
